#!/usr/bin/env ruby

##
# cs50:fi submission script
#
# Tommy MacWilliam <tmacwilliam@cs.harvard.edu>
#
#

require 'net/https'
require 'uri'
require 'optparse'

$URL_BASE = "repo.cs50.net"

# make sure stty is re-enabled
trap("SIGINT") do
	`stty echo`
	puts ""
	exit!
end

course = ""
username = ""
project = ""
password = ""
verbose = false
very_verbose = false

OptionParser.new do |opts|
	# course flag
	opts.on("-c", "--course [COURSE]", "Course") do |c|
		course = c
	end

	# username flag
	opts.on("-u", "--username [USERNAME]", "Git username") do |u|
		username = u
	end

	# repository flag
	opts.on("-r", "--repo [REPOSITORY]", "Project repository") do |r|
		project = r
	end

	# verbose flag
	opts.on("-v", "--verbose", "Verbose output") do |v|
		verbose = v
	end

	# very verbose flag
	opts.on("-vv", "--very-verbose", "Very verbose output") do |v|
		very_verbose = v
	end
end.parse!

# path is requried argument
if (ARGV.length != 1)
	abort "Usage: submit /path/to/submit"
end
path = ARGV[0]

# prompt for course
while (course == nil || course.empty?)
	print "Course: "
	course = STDIN.gets
	course = course.strip unless course == nil
end

# prompt for project
while (project == nil || project.empty?)
	print "Repository: "
	project = STDIN.gets
	project = project.strip unless project == nil
end

# prompt for username
while (username == nil || username.empty?)
	print "Username: "
	username = STDIN.gets
	username = username.strip unless username == nil
end

# prompt user for password, but don't output while typing
while (password == nil || password.empty?)
	`stty -echo`
	print "Password: "
	password = STDIN.gets
	password = password.strip unless password == nil
	`stty echo`
	puts ""
end

if (!File.directory?(path))
	abort "Please specify the path to the folder containing your project."
end

# change to directory user specified
Dir.chdir(path) do
	# create a new git repo if one doesn't already exist
	if (!File.directory?(".git"))
		puts "Creating git repository..." if verbose
		`git init #{unless very_verbose then '--quiet > /dev/null 2>&1' end}`
	end

	# add and commit entire directory
	puts "Committing changes..." if verbose
	`git add --all #{unless very_verbose then '> /dev/null 2>&1' end}`
	`git commit -m 'submit' #{unless very_verbose then '> /dev/null 2>&1' end}`

	# tag commit as submission, replacing any previous submission
	puts "Tagging changes as submission..." if verbose
	`git tag submit --force #{unless very_verbose then '> /dev/null 2>&1' end}`

	# push branch to server
	puts "Pushing changes..." if verbose
	git_url = "https://#{username}:#{password}@#{$URL_BASE}/git/#{course}/#{project}/#{username}.git"
	`git push #{git_url} master #{unless very_verbose then '--quiet > /dev/null 2>&1' end}`

	# push failed as a result of non-fast-forward or invalid credentials
	if ($?.exitstatus != 0)
		puts "Push rejected, cloning existing repository..." if verbose
		# clone remote into a temporary directory
		`rm -rf .git`
		tmp_path = "/tmp/#{course}/#{project}/#{username}"
		`rm -rf #{tmp_path}`
		`mkdir -p #{tmp_path}`
		`git clone #{git_url} #{tmp_path} #{unless very_verbose then '--quiet > /dev/null 2>&1 ' end}`
	    
		# clone failed, so user must have supplied invalid credentials
		if ($?.exitstatus != 0)
			abort "Your project has NOT been submitted. Your Course, Repository\nUsername, and/or Password were incorrect. Please try again."
		end

		# clone succeeded, so replace repo with new submit contents
		puts "Clone successful, re-pushing..." if verbose
		`rm -rf #{tmp_path}/*`
		`cp -r * #{tmp_path}`

		# submit from temp directory
		Dir.chdir(tmp_path) do
			`git add --all #{unless very_verbose then '> /dev/null 2>&1' end}`
			`git commit -m 'submit' #{unless very_verbose then '> /dev/null 2>&1' end}`
			`git tag submit --force #{unless very_verbose then '> /dev/null 2>&1' end}`
			`git push #{git_url} master #{unless very_verbose then '--quiet > /dev/null 2>&1' end}`
			`git push #{git_url} --tags #{unless very_verbose then '--quiet > /dev/null 2>&1' end}`
		end

		# move the temp .git directory into the working directory to avoid /tmp submission again
		`cp -r #{tmp_path}/.git .`
		`rm -rf #{tmp_path}`

	# push was successful, so proceed to push tags
	else
		`git push #{git_url} --tags #{unless very_verbose then '--quiet > /dev/null 2>&1' end}`
	end

	puts "Changes pushed!" if verbose
	puts "Your work HAS been submitted!"

	# check if PDF was rendered
	pdf_url = "https://#{$URL_BASE}/pdf/#{course}/#{project}/#{username}.git/submit.pdf"
	uri = URI.parse(pdf_url)

	# only show PDF url if PDF was actually rendered
	begin
		http = Net::HTTP.new(uri.host, uri.port)
		request = Net::HTTP::Get.new(uri.path)
		http.use_ssl = true
		http.verify_mode = OpenSSL::SSL::VERIFY_NONE
		request.basic_auth(username, password)
		response = http.request(request)
		response.value()
	rescue
		exit
	end

	puts "To view your submission, visit #{pdf_url}"
end
